/*------------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich & W.Wallucks
 Remarks:        
 known Problems: none
 Version:        28.04.2008
 Description:    Command Interpreter

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
-------------------------------------------------------------------------------*/

//----------------------------------------------------------------------------
#include "../config.h"

#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>
#include <avr/io.h>
#include <avr/eeprom.h>
#include <string.h>
#include <avr/pgmspace.h>
#include <util/delay.h>

#if TCP_SERVICE

#include "tcpcmd.h"
#include "tcpsrv.h"

#include "../usart.h"
#include "../stack.h"
#include "../ntp.h"
#include "../wol.h"
#include "../sd-reader_source_20071213/fat16.h"
#include "../sd-reader_source_20071213/sd_raw.h"
#include "../sdkarte/sdcard.h"
#include "../sendmail.h"



/*
**	lokale Variable
*/
volatile unsigned int variable[MAX_VAR];
char *argv;

extern struct fat16_fs_struct* g_fs;	// globaler Zeiger auf FAT16 Struktur der SD-Karte
extern TCPSRV_STATUS tcpsrv_status;

// Prototypen
uint8_t getInteger(char *);
int16_t cat(char *);
int16_t cmd_PWD(char *);
int16_t cmd_CWD(char *);
int16_t cmd_CDUP(char *);
int16_t cmd_150(char *);
int16_t cmd_200(char *);
int16_t cmd_250(char *);
int16_t cmd_PASV(char *);
int16_t cmd_502(char *);
int16_t cmd_550(char *);
int16_t cmd_TYPE(char *);
int16_t cmd_RETR(char *);
int16_t cmd_LIST(char *);
int16_t cmd_STOR(char *);
int16_t cmd_DELE(char *);
int16_t cmd_RMD(char *);
int16_t cmd_MKD(char *);
int16_t cmd_quit(char *);


CMD_ITEM CMD_TABLE[] = // Befehls-Tabelle
{
	{"QUIT",cmd_quit},
	{"EXIT",cmd_quit},
	{"RESET",command_reset},
	{"ARP",command_arp},
	{"TCP",command_tcp},
	{"IP",command_ip},
	{"NET",command_net},
	{"ROUTER",command_router},
	{"NTP",command_ntp},
	{"MAC",command_mac},
	{"VER",command_ver},
	{"SV",command_setvar},
	{"TIME",command_time},
	{"NTPR",command_ntp_refresh},	
#if USE_WOL
	{"WOL",command_wol},
#endif

#if USE_MMC
	{"DIR",cmd_MMCdir},
	{"DISK",print_disk_info},
	{"CAT",cat},
	{"PWD",cmd_PWD},
	{"CWD",cmd_CWD},
	{"CD",cmd_CWD},
	{"CDUP",cmd_CDUP},
	{"USER",cmd_200},
	{"NOOP",cmd_200},
	{"TYPE",cmd_TYPE},
	{"PASV",cmd_PASV},
	{"LIST",cmd_LIST},
	{"RETR",cmd_RETR},
	{"STOR",cmd_STOR},
	{"RMD",cmd_DELE},
	{"MKD",cmd_MKD},
	{"DELE",cmd_DELE},
	{"NOOP",cmd_200},
#endif

	{"PING", command_ping},
	#if USE_MAIL
    {"mail", command_mail},
    #endif //USE_MAIL
#if HELPTEXT
	{"HELP",command_help},
	{"?",command_help},
#endif
	{"ADC",command_ADC},

	{NULL,NULL} 
};

PROGMEM char OK_txt[] = { "+OK\r\n"};
#define OK_LEN	5

#if HELPTEXT
	PROGMEM char helptext[] = {
		"RESET  - reset the AVR - Controller\r\n"
		"ARP    - list the ARP table\r\n"
		"TCP    - list the tcp table\r\n"
		"IP     - list/change ip\r\n"
		"NET    - list/change netmask\r\n"
		"ROUTER - list/change router ip\r\n"
		"NTP    - list/change NTP\r\n"
		"NTPR   - NTP Refresh\r\n"
		"MAC    - list MAC-address\r\n"
		"VER    - list enc version number\r\n"
		"SV     - set variable\r\n"
        "PING   - send Ping\r\n"
        #if USE_MAIL
        "MAIL   - send E-Mail\r\n"
        #endif //USE_MAIL
		#if USE_WOL
		"WOL    - send WOL / set MAC / set MAC and IP\r\n"
		#endif
		#if USE_MMC
		"disk   - SD-Karteninformation\r\n"
		"dir    - Directory der SD-Karte\r\n"
		"cat    - Datei anzeigen\r\n"
		"cwd    - change working directory\r\n"
		"cd     - wie cwd\r\n"
		#endif
		"TIME   - get time\r\n"
		"HELP   - print Helptext\r\n"
		"?      - print Helptext\r\n"
	};
#endif

//------------------------------------------------------------------------------
//Commando auswerten
unsigned char extract_cmd(char *pstr)
{
	//Stringzeiger;
	char *string_pointer_tmp;
	unsigned char cmd_index = 0;
 
    string_pointer_tmp = strsep(&pstr," "); 
	argv = pstr;	// Zeiger auf Cmd-Parameter setzen

	//Kommando in Tabelle suchen
	while(strcasecmp(CMD_TABLE[cmd_index].cmd,string_pointer_tmp))
    {
        //Abruch Whileschleife und Unterprogramm verlassen 
        if (CMD_TABLE[++cmd_index].cmd == 0) return(0);
    }
    
	usart_write("\r\n");
	CMD_TABLE[cmd_index].fp(0); // Befehl ausfhren und Ergebnis zurckgeben
	return(1); 
}

/*
*	getInteger - Integer-Werte von CMD-Parameter in Variable einlesen
*/
uint8_t getInteger(char *pstr)
{
	char *ps;
	uint8_t i = 0;

	for (uint8_t n = 0; n<MAX_VAR;++n) {
		ps = strsep(&pstr,".");
		variable[n] = strtol(ps,NULL,0);
		++i;
	}

	return i;	// Anzahl konvertierter Zahlen
}

//------------------------------------------------------------------------------
// (-1) fr EXIT/QUIT zurckgeben
int16_t cmd_quit(char *outbuffer)
{
	return (-1);
}
//------------------------------------------------------------------------------
//Reset ausfhren
int16_t command_reset (char *outbuffer)
{
	RESET();
	return 0;	// never executed
}

//------------------------------------------------------------------------------
//print/edit own IP
int16_t command_ip (char *outbuffer)
{
	write_eeprom_ip(IP_EEPROM_STORE);
	(*((unsigned long*)&myip[0])) = get_eeprom_value(IP_EEPROM_STORE,MYIP);
	if (outbuffer) {
		sprintf_P(outbuffer,PSTR("My IP: %i.%i.%i.%i\r\n"),myip[0],myip[1],myip[2],myip[3]);
		return strlen(outbuffer);
	}
	else {
		usart_write("My IP: %1i.%1i.%1i.%1i\r\n",myip[0],myip[1],myip[2],myip[3]);
		return 0;
	}
}

//------------------------------------------------------------------------------
//
void write_eeprom_ip (unsigned int eeprom_adresse)
{
	getInteger((char *)argv);	// Zahlen in Parameterliste nach variable[] speichern
	if (*((unsigned int*)&variable[0]) != 0x00000000)
	{
		//value ins EEPROM schreiben
		for (unsigned char count = 0; count<4;count++)
		{
			eeprom_busy_wait ();
			eeprom_write_byte((unsigned char *)(eeprom_adresse + count),variable[count]);
		}
	}
}

//------------------------------------------------------------------------------
//print/edit NTP Server IP
int16_t command_ntp (char *outbuffer)
{
	#if USE_NTP
	write_eeprom_ip(NTP_IP_EEPROM_STORE);
	(*((unsigned long*)&ntp_server_ip[0])) = get_eeprom_value(NTP_IP_EEPROM_STORE,NTP_IP);
	if (outbuffer) {
		sprintf_P(outbuffer,PSTR("NTP_Server: %i.%i.%i.%i\r\n"),ntp_server_ip[0],ntp_server_ip[1],ntp_server_ip[2],ntp_server_ip[3]);
		return strlen(outbuffer);
	}
	else {
		usart_write("NTP_Server: %1i.%1i.%1i.%1i\r\n",ntp_server_ip[0],ntp_server_ip[1],ntp_server_ip[2],ntp_server_ip[3]);
		return 0;
	}
	#endif //USE_NTP
}

//------------------------------------------------------------------------------
//print/edit Netmask
int16_t command_net (char *outbuffer)
{
	write_eeprom_ip(NETMASK_EEPROM_STORE);
	(*((unsigned long*)&netmask[0])) = get_eeprom_value(NETMASK_EEPROM_STORE,NETMASK);
	if (outbuffer) {
		sprintf_P(outbuffer,PSTR("NETMASK: %i.%i.%i.%i\r\n"),netmask[0],netmask[1],netmask[2],netmask[3]);
		return strlen(outbuffer);
	}
	else {
		usart_write("NETMASK: %1i.%1i.%1i.%1i\r\n",netmask[0],netmask[1],netmask[2],netmask[3]);
		return 0;
	}
}

//------------------------------------------------------------------------------
//print/edit Router IP
int16_t command_router (char *outbuffer)
{
	write_eeprom_ip(ROUTER_IP_EEPROM_STORE);
	(*((unsigned long*)&router_ip[0])) = get_eeprom_value(ROUTER_IP_EEPROM_STORE,ROUTER_IP);
	if (outbuffer) {
		sprintf_P(outbuffer,PSTR("250 ok. Router IP: %i.%i.%i.%i\r\n"),router_ip[0],router_ip[1],router_ip[2],router_ip[3]);
		return strlen(outbuffer);
	}
	else {
		usart_write("Router IP: %1i.%1i.%1i.%1i\r\n",router_ip[0],router_ip[1],router_ip[2],router_ip[3]);
		return 0;
	}
}

//------------------------------------------------------------------------------
//print own mac
int16_t command_mac (char *outbuffer)
{
	if (outbuffer) {
		sprintf_P(outbuffer,PSTR("250 ok. My MAC: %2x:%2x:%2x:%2x:%2x:%2x\r\n"),mymac[0],mymac[1],mymac[2],mymac[3],mymac[4],mymac[5]);
		return strlen(outbuffer);
	}
	else {
		usart_write("My MAC: %2x:%2x:%2x:%2x:%2x:%2x\r\n",mymac[0],mymac[1],mymac[2],mymac[3],mymac[4],mymac[5]);
		return 0;
	}
}

//------------------------------------------------------------------------------
//print enc28j60 chip version
int16_t command_ver (char *outbuffer)
{
#if USE_ENC28J60
	if (outbuffer) {
		sprintf_P(outbuffer,PSTR("250 ok. ENC28J60-Version: %1x\r\n"), enc28j60_revision);
		return strlen(outbuffer);
	}
	else {
		usart_write("ENC28J60-Version: %1x\r\n", enc28j60_revision);
		return 0;
	}
#endif

#if USE_RTL8019
	usart_write("RTL8019 Ethernetcard\r\n");
#endif
}

//------------------------------------------------------------------------------
//print ARP table
extern struct arp_table arp_entry[MAX_ARP_ENTRY];

int16_t command_arp (char *outbuffer)
{
	if (outbuffer)					// momentan nur bei USART
		return cmd_502(outbuffer);

	for (unsigned char index = 0;index<MAX_ARP_ENTRY;index++)
	{
		usart_write("%2i  MAC:%2x",index,arp_entry[index].arp_t_mac[0]);
		usart_write(".%2x",arp_entry[index].arp_t_mac[1]);
		usart_write(".%2x",arp_entry[index].arp_t_mac[2]);
		usart_write(".%2x",arp_entry[index].arp_t_mac[3]);
		usart_write(".%2x",arp_entry[index].arp_t_mac[4]);
		usart_write(".%2x",arp_entry[index].arp_t_mac[5]);
		
		usart_write("  IP:%3i",(arp_entry[index].arp_t_ip&0x000000FF));
		usart_write(".%3i",((arp_entry[index].arp_t_ip&0x0000FF00)>>8));
		usart_write(".%3i",((arp_entry[index].arp_t_ip&0x00FF0000)>>16));
		usart_write(".%3i",((arp_entry[index].arp_t_ip&0xFF000000)>>24));
			
		usart_write("  Time:%4i\r\n",arp_entry[index].arp_t_time);
	}
	return 0;
}

//------------------------------------------------------------------------------
//print TCP table
extern struct tcp_table tcp_entry[MAX_TCP_ENTRY+1];

int16_t command_tcp (char *outbuffer)
{
	if (outbuffer)					// momentan nur bei USART
		return cmd_502(outbuffer);

	for (unsigned char index = 0;index<MAX_TCP_ENTRY;index++)
	{
		usart_write("%2i",index);
		usart_write("  IP:%3i",(tcp_entry[index].ip&0x000000FF));
		usart_write(".%3i",((tcp_entry[index].ip&0x0000FF00)>>8));
		usart_write(".%3i",((tcp_entry[index].ip&0x00FF0000)>>16));
		usart_write(".%3i",((tcp_entry[index].ip&0xFF000000)>>24));
		usart_write("  PORT:%4i",LBBL_ENDIAN_INT(tcp_entry[index].src_port));
		usart_write("  Time:%4i\r\n",tcp_entry[index].time);
	}
	return 0;
}


//------------------------------------------------------------------------------
//ndern einer Variable
int16_t command_setvar (char *outbuffer)
{
#if USE_ADC
	getInteger(argv);	// Zahlen in Parameterliste nach variable[] speichern
	var_array[variable[0]] = variable[1];
	if (outbuffer) {
		sprintf_P(outbuffer,PSTR("Inhalt der Variable[%2i] = %2i\r\n"),variable[0],var_array[variable[0]]);
		return strlen(outbuffer);
	}
	else {
		usart_write("Inhalt der Variable[%2i] = %2i\r\n",variable[0],var_array[variable[0]]);
	}
#endif
	return 0;
}

//------------------------------------------------------------------------------
//print Time
int16_t command_time (char *outbuffer)
{
	unsigned char hh = (time/3600)%24;
	unsigned char mm = (time/60)%60;
	unsigned char ss = time %60;
	if (outbuffer) {
		sprintf_P(outbuffer,PSTR("250 ok. Time: %2i:%2i:%2i\r\n"),hh,mm,ss);
		return strlen(outbuffer);
	}
	else {
		usart_write ("\n\rTIME: %2i:%2i:%2i\r\n",hh,mm,ss);
		return 0;
	}
}

//------------------------------------------------------------------------------
//Time Refresh via NTP-Server
int16_t command_ntp_refresh (char *outbuffer)
{
	#if USE_NTP
	ntp_request();
	#endif //USE_NTP
	return cmd_250(outbuffer);
}

//------------------------------------------------------------------------------
//Sendet eine fertige E-MAIL
#if USE_MAIL
int16_t command_mail (char *outbuffer)
{
	mail_enable = 1;
	return cmd_250(outbuffer);
}
#endif //USE_MAIL

//------------------------------------------------------------------------------
//
int16_t command_wol (char *outbuffer)
{
	if (outbuffer)					// nur bei USART
		return cmd_502(outbuffer);

	#if USE_WOL
	getInteger(argv);	// Zahlen in Parameterliste nach variable[] speichern
	// EEPROM beschreiben, falls Parameter angegeben wurden
	if ((*((unsigned int*)&variable[0]) != 0x00000000) || (*((unsigned int*)&variable[1]) != 0x00000000) || (*((unsigned int*)&variable[2]) != 0x00000000))
	{	
		//schreiben der MAC
		for (unsigned char count = 0; count<6; count++)
		{
			eeprom_busy_wait ();
			eeprom_write_byte((unsigned char *)(WOL_MAC_EEPROM_STORE + count),variable[count]);
		}
		//zustzlich schreiben der Broadcast-Adresse, falls vorhandenden
		for (unsigned char count = 0; count<4 && (*((unsigned int*)&variable[6]) != 0x00000000);count++)
		{
			eeprom_busy_wait ();
			eeprom_write_byte((unsigned char*)(WOL_BCAST_EEPROM_STORE + count),variable[count+6]);
		}
		//init
		wol_init();
	}else{
		//MagicPacket senden
		wol_request();
	}
	#endif //USE_WOL	
	return 0;
}


//------------------------------------------------------------------------------
int16_t cmd_MMCdir(char *outbuffer)
{
#if USE_MMC
	if (!f16_init())
		return 0;

    /* open root directory */
    struct fat16_dir_entry_struct directory;
    fat16_get_dir_entry_of_path(g_fs, cwdirectory, &directory);

    struct fat16_dir_struct* dd = fat16_open_dir(g_fs, &directory);
    if(!dd) {
        return 0;
    }

    /* print directory listing */
    struct fat16_dir_entry_struct dir_entry;

	if (!outbuffer) {
		usart_write("\n\rSD-Karte:\\>");
	}

	char *pstr = outbuffer;
	uint16_t year;
	uint8_t month;
	uint8_t day;
	uint8_t hour;
	uint8_t min;
	uint8_t sec;

    while(fat16_read_dir(dd, &dir_entry))
    {
		fat16_get_file_modification_date(&dir_entry, &year, &month, &day);
		fat16_get_file_modification_time(&dir_entry, &hour, &min, &sec);

		if (outbuffer) {
			char tmpbuf[10];
			if (dir_entry.attributes & FAT16_ATTRIB_DIR)
				strcpy_P((char *)tmpbuf,PSTR("<DIR>"));
			else
				sprintf_P(tmpbuf,PSTR("%ld"),dir_entry.file_size);

			sprintf_P((char *)pstr,PSTR("%i.%i.%i %i:%i %8s %s\r\n"),day, month, year, hour, min, tmpbuf,dir_entry.long_name);

			while (*pstr++);	// bis auf '\0' vorzhlen
			--pstr;
		}
		else {
			usart_write("\r\n%2i.%2i.%i %2i:%2i ",day, month, year, hour, min);
			if (dir_entry.attributes & FAT16_ATTRIB_DIR)
				usart_write("    <DIR> ");
			else
				usart_write("%9l ",dir_entry.file_size);

			usart_write("%s",dir_entry.long_name);
		}
    }

	fat16_close_dir(dd);

	if (outbuffer)
		return strlen(outbuffer);
	else
		return 0;
#endif
}

int16_t cat(char *outbuffer)
{
	if (outbuffer)					// nur bei USART
		return cmd_502(outbuffer);

	int ch;

	File *ptrFile = f16_open(argv,"r");
	if (!ptrFile) return 0;

	ch = f16_getc(ptrFile);
	while ( ch > 0 ) {
		usart_write_char((char)ch);
		ch = f16_getc(ptrFile);
		}

	f16_close(ptrFile);
	return 0;
}

int16_t cmd_RETR(char *outbuffer)
{
	if (outbuffer) {
		if( f16_exist(argv) ) {
			strcpy((char *)tcpsrv_status.fname,argv);
			tcpsrv_status.data_state = 2;
			// Anfang der bertragung
			return cmd_150(outbuffer);
		}
		else
			cmd_502(outbuffer);	// TODO richtige Fehlermeldung einsetzen
	}
	return 0;
}

int16_t cmd_LIST(char *outbuffer)
{
	if (outbuffer) {
		tcpsrv_status.data_state = 1;
		// Anfang der bertragung
		return cmd_150(outbuffer);
	}
	return 0;
}

int16_t cmd_STOR(char *outbuffer)
{
	if (outbuffer) {
		if( f16_exist(argv) ) {
			// Datei lschen
			f16_delete(argv);
		}

		strcpy((char *)tcpsrv_status.fname,argv);
		tcpsrv_status.data_state = 3;
		// Anfang der bertragung
		return cmd_150(outbuffer);
	}
	return 0;
}

int16_t cmd_DELE(char *outbuffer)
{
	if( f16_exist(argv) ) {
		if (f16_delete(argv) && outbuffer)
				return cmd_200(outbuffer);	// success
	}
	if (outbuffer) {
		return cmd_550(outbuffer);
	}
	return 0;
}

int16_t cmd_MKD(char *outbuffer)
{
	if( f16_mkdir(argv) && outbuffer) {
		return cmd_200(outbuffer);	// success
	}

	if (outbuffer)
		return cmd_550(outbuffer);
	else
		return 0;
}

int16_t cmd_PWD(char *outbuffer)
{
	if (outbuffer) {
		sprintf_P(outbuffer,PSTR("257 \"%s\" is your current location\r\n"),cwdirectory);
		return strlen(outbuffer);
	}
	else {
		return 0;
	}
}

/*
*	CWD bzw. CD Befehl
*	neuen Pfad in cwdirectory abspeichern
*	und auf Existenz berprfen
*	- cwdirectory enthlt den aktuellen Pfad ohne trailing "/"
*	  nur das Rootverzeichnis enthlt "/"
*/
int16_t cmd_CWD(char *outbuffer)
{
	if (*argv != '/') {								// bei aktuellem directory beginnen

		char tmpdir[MAX_PATH+1];
		strcpy(tmpdir,cwdirectory);

		if ((strlen(argv) == 2) && *argv == '.' && *(argv+1) == '.') {
			strcat_P(argv,PSTR("/"));
		}

		while ((strlen(argv) > 2) && *argv == '.' && *(argv+1) == '.' && *(argv+2) == '/') {
			// ein Verzeichnis nach oben
			int8_t i = strlen(tmpdir) - 1;

			for (;i>=0;--i) {
				if (tmpdir[i] == '/') {
					tmpdir[i] = '\0';
					break;
				}
			}

			if (i <= 0) {
				strcpy_P(tmpdir,PSTR("/"));	// oben angekommen
			}

			argv += 3;
		}

		if (*argv == '.' && *(argv+1) == '/') argv += 2;	// zeigt auf aktuelles Verzeichnis

		uint8_t len = strlen(tmpdir);
		if ( len > 1)
			strcat_P(tmpdir,PSTR("/"));

		if ((len + strlen(argv)) < MAX_PATH) {	// falls der Pfad auch in den Speicherplatz passt

			register char *ptr = argv;	// letzen '/' entfernen
			while (*ptr++);
			ptr -= 2;
			if (*ptr == '/') *ptr = '\0';

			strcat(tmpdir,argv);

		    struct fat16_dir_entry_struct directory;
    		if (fat16_get_dir_entry_of_path(g_fs, tmpdir, &directory)) {
				strcpy((char *)cwdirectory,tmpdir);
			}
		}
	}
	else
		strncpy((char *)cwdirectory,argv,MAX_PATH);

	if (outbuffer) {
		sprintf_P(outbuffer,PSTR("250 Directory changed to %s\r\n"),cwdirectory);
		return strlen(outbuffer);
	}
	else {
		return 0;
	}
}

int16_t cmd_CDUP(char *outbuffer)
{
	// ein Verzeichnis nach oben
	int8_t i = strlen(cwdirectory) - 1;
	char *pdir = (char *)cwdirectory;

	for (;i>=0;--i) {
		if (*(pdir+i) == '/') {
			*(pdir+i) = '\0';
			break;
		}
	}

	if (i <= 0) {
		strcpy_P((char *)cwdirectory,PSTR("/"));	// oben angekommen
	}

	if (outbuffer) {
		sprintf_P(outbuffer,PSTR("250 Directory changed to %s\r\n"),cwdirectory);
		return strlen(outbuffer);
	}
	else {
		return 0;
	}
}

int16_t cmd_TYPE(char *outbuffer)
{
	if (outbuffer) {
		if (*argv == 'I') {
			strcpy_P(outbuffer,PSTR("200 Using BINARY mode to transfer data.\r\n"));
			tcpsrv_status.transfermode = 1;
		}
		else {
			strcpy_P(outbuffer,PSTR("200 Using ASCII mode to transfer data.\r\n"));
			tcpsrv_status.transfermode = 0;
		}
		return strlen(outbuffer);
	}
	else {
		return 0;
	}
}

int16_t cmd_150(char *outbuffer)
{
	if (outbuffer) {
		if (tcpsrv_status.transfermode)
			strcpy_P(outbuffer,PSTR("150 Opening BINARY mode data connection.\r\n"));
		else
			strcpy_P(outbuffer,PSTR("150 Opening ASCII mode data connection.\r\n"));
		return strlen(outbuffer);
	}
	else {
		return 0;
	}
}

int16_t cmd_200(char *outbuffer)
{
	if (outbuffer) {
		strcpy_P(outbuffer,PSTR("200 Command ok.\r\n"));
		return strlen(outbuffer);
	}
	else {
		return 0;
	}
}

int16_t cmd_250(char *outbuffer)
{
	if (outbuffer) {
		strcpy_P(outbuffer,PSTR("250 ok.\r\n"));
		return strlen(outbuffer);
	}
	else {
		return 0;
	}
}

int16_t cmd_502(char *outbuffer)
{
	if (outbuffer) {
		strcpy_P(outbuffer,PSTR("502 not implemented.\r\n"));
		return strlen(outbuffer);
	}
	else {
		return 0;
	}
}

int16_t cmd_550(char *outbuffer)
{
	if (outbuffer) {
		strcpy_P(outbuffer,PSTR("550 Requested action not taken.\r\n"));
		return strlen(outbuffer);
	}
	else {
		return 0;
	}
}

/*
*	PASV passiver mode auf Port 2100 (8 * 256 + 52)
*/
int16_t cmd_PASV(char *outbuffer)
{
	if (outbuffer) {
		sprintf_P(outbuffer,PSTR("227 Entering Passive Mode (%i,%i,%i,%i,8,52)\r\n"),myip[0],myip[1],myip[2],myip[3]);
		return strlen(outbuffer);
	}
	else {
		return 0;
	}
}


//------------------------------------------------------------------------------
int16_t command_ADC(char *outbuffer)
{
	if (outbuffer)					// nur bei USART
		return cmd_502(outbuffer);

	int8_t	i;

	for (i=0; i<MAX_VAR_ARRAY; i++) {
		usart_write("\r\nADC-Kanal(%i) = %i",i,var_array[i]);
	}
	usart_write("\r\n");
	return 0;
}

//------------------------------------------------------------------------------
// Sende "Ping" an angegebene Adresse
int16_t command_ping (char *outbuffer)
{
	getInteger(argv);	// Zahlen in Parameterliste nach variable[] speichern
	if (*((unsigned int*)&variable[0]) != 0x00000000)
	{
		unsigned long dest_ip = ((variable[0])+((unsigned long)(variable[1])<<8)+((unsigned long)(variable[2])<<16)+((unsigned long)(variable[3])<<24));
		
		//ARP Request senden
		arp_request (dest_ip);
		
		//ICMP-Nachricht Type=8 Code=0: Echo-Anfrage
		//TODO: Sequenznummer, Identifier 
		icmp_send(dest_ip,0x08,0x00,1,1);
	}
	return 0;
}

//------------------------------------------------------------------------------
//print helptext
int16_t command_help (char *outbuffer)
{
	if (outbuffer)					// nur bei USART
		return cmd_502(outbuffer);

#if HELPTEXT
	unsigned char data;
	PGM_P helptest_pointer = helptext;
	
	do
	{
		data = pgm_read_byte(helptest_pointer++);
		usart_write("%c",data);
	}while(data != 0);
	return 0;
#endif
}

int16_t print_disk_info(char *outbuffer)	//const struct fat16_fs_struct* fs)
{
	if (!f16_init())
        return 0;

	if (outbuffer)					// nur bei USART
		return cmd_502(outbuffer);

    struct sd_raw_info disk_info;
    if(!sd_raw_get_info(&disk_info)) {
	    usart_write("\r\nDisk kann nicht gelesen werden.");
        return 0;
	}

    usart_write("\r\nmanuf: 0x%x",disk_info.manufacturer);
    usart_write("\r\noem:     %s",disk_info.oem);
    usart_write("\r\nprod:    %s",disk_info.product);
    usart_write("\r\nrev:     %i",disk_info.revision);
    usart_write("\r\nserial:  %l",disk_info.serial);
    usart_write("\r\ndate:    %i/%i",disk_info.manufacturing_month,disk_info.manufacturing_year);
    usart_write("\r\nsize:    %l",disk_info.capacity);
    usart_write("\r\ncopy:    %i",disk_info.flag_copy);
    usart_write("\r\nwr.pr.:  %i/%i",disk_info.flag_write_protect_temp,disk_info.flag_write_protect);
    usart_write("\r\nformat:  %i",disk_info.format);
//    uart_puts_p(PSTR("free:   ")); uart_putdw_dec(fat16_get_fs_free(fs)); uart_putc('/');
//                                   uart_putdw_dec(fat16_get_fs_size(fs)); uart_putc('\n');
	return 0;
}

#endif
