/*----------------------------------------------------------------------------
 Copyright:      W.Wallucks
 known Problems: ? // siehe TODO
 Version:        28.04.2008
 Description:    SD-Kartenanbindung an AVR-Miniwebserver

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
----------------------------------------------------------------------------*/
#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <avr/io.h>
#include <avr/pgmspace.h>

#include "../config.h"
#include "cmd.h"

#include "tcpsrv.h"

#include "../stack.h"
#include "../usart.h"

/***********************
* 	Vars
*/

#define CMD_TABLE COMMAND_TABELLE

TCPSRV_STATUS tcpsrv_status;
char *argv;
uint16_t length;
uint8_t index_temp;



//----------------------------------------------------------------------------
//Initialisierung des Servers
void tcpsrvd_init(void)
{
	//Serverport und Anwendung in TCP Anwendungsliste eintragen
	add_tcp_app (21, (void(*)(unsigned char))tcpsrvd);
//	add_tcp_app (2100, (void(*)(unsigned char))datachannel);
	add_tcp_app (MYTCP_PORT, (void(*)(unsigned char))tcpsrvd);
}


/*
*	getInteger - Integer-Werte von CMD-Parameter in Variable einlesen
*/
uint8_t getInteger(char *pstr)
{
	char *ps;
	uint8_t i = 0;

	for (uint8_t n = 0; n<MAX_VAR;++n) {
		ps = strsep(&pstr,". ");
		variable[n] = strtol(ps,NULL,0);
		++i;
	}

	return i;	// Anzahl konvertierter Zahlen
}

void eth_print(char * ptr)
{
	strcpy((char *)&eth_buffer[TCP_DATA_START],ptr);
	length=strlen((char *)&eth_buffer[TCP_DATA_START]);
}


uint16_t doSrvCmd(uint8_t index, unsigned char *cmdstr)
{
	int16_t len = 0;
	uint8_t i = 0;

	// Lngenzhler zwechs Ausgabe ber TCP - nicht schn, aber geht.
	length = 0;

	while (CMD_TABLE[i].cmd && strcasecmp(CMD_TABLE[i].cmd,(char *)cmdstr)) i++;
	// Wenn Befehlsindex zu gross, dann mit Fehler beenden
	if (!CMD_TABLE[i].cmd) {
		strcpy_P((char *)&eth_buffer[TCP_DATA_START],PSTR("502 not implemented.\r\n"));
		return strlen((char *)&eth_buffer[TCP_DATA_START]);
	}

	argv = (char *)tcpsrv_status.argvbuffer;	// Command-Parameter setzen

 	// Variablen einlesen
	getInteger(argv);

	//strcpy_P((char *)&eth_buffer[TCP_DATA_START],PSTR("Hhnelgagelei.\r\n"));
	//len=strlen((char *)&eth_buffer[TCP_DATA_START]);
	
	CMD_TABLE[i].fp();	
	len = length; // in der aufgerufenen Funktion gab es evtl. eine Ausgabe,
				  // welche length gesetzt hat - diese muss nun zwecks Ausgabe
				  // via TCP an len bergeben werden, um dann in respond aus-
				  // gegeben zu werden.
	length = 0;
	
	if (len < 0) {	// negativer Wert beendet Verbindung
		//tcp_entry[index].status = ACK_FLAG | FIN_FLAG;
		//tcp_entry[index].app_status = 0xFFFD;	// 123456789.123456789.123456789.
		memcpy_P(&eth_buffer[TCP_DATA_START],PSTR("bye, bye ...\r\n"),14);
		tcp_Port_close(index);
		len = 14;
	}
	return (len);
}

//----------------------------------------------------------------------------
//tcp Server (CTRL-Channel)
void tcpsrvd(uint8_t index)
{
	TCPSRV_DEBUG("\r\n---- TCP Server-CTRL %i gestartet. ---- Flags: 0x%x",index,tcp_entry[index].status);

	if (tcp_entry[index].app_status <= 1) {	// erstmaliger Aufruf der Anwendung -> Begrung senden
		tcp_entry[index].app_status = 1;
		respond_P(CTRL_CHANNEL, PSTR("220 Server bereit\r\n"),index);
		tcpsrv_status.transfermode = 0;	// ASCII mode
		tcpsrv_status.tcpindex = index;	// letzten socket merken
		return;
	}
	// else if(tcp_entry[index].app_status > 1) ...

	if (tcp_entry[index].status & FIN_FLAG) {	// Wird Verbindungsabbau signalisiert?
		TCPSRV_DEBUG("\r\nCTRL-channel %i closed.",index);
		tcpsrv_status.tcpindex = 0xff;
		return;
	}

	if (tcp_entry[index].status&PSH_FLAG) {		// Ist das Paket fr die Anwendung bestimmt ?
		tcp_entry[index].app_status = 2;		// nicht automatisch weiterzhlen

		/*
		* eingehende Zeichen (bei TELNET 1 char pro Paket !) sammeln
		* und Cmd-Zeile interpretieren.
		* Cmd und Parameter getrennt erfassen.
		*/
		for (int a = TCP_DATA_START_VAR;a<(TCP_DATA_END_VAR);a++) {	// Schleife ber alle Zeichen im Empfangspuffer

			if (eth_buffer[a] == '\r') {			// Zeilenende erkannt ?

				TCPSRV_DEBUG("\r\nTCP Server-CTRL cmd: %s argv: %s",tcpsrv_status.cmdbuffer,tcpsrv_status.argvbuffer);

				tcpsrv_status.tcpindex = index; 	// letzten socket merken

				// ausfhren und Ergebnis im TCP-Paket zurckgeben
				// AR index merken
				index_temp=index;

				_respond(CTRL_CHANNEL, doSrvCmd(index,(unsigned char *)tcpsrv_status.cmdbuffer),index);
				return;
			}

			else if ( tcpsrv_status.read_argv == 0 && eth_buffer[a] == ' ') {	// cmd Ende, Parameter folgt
				tcpsrv_status.read_argv = 1;
			}

			else if (eth_buffer[a] >= ' ') {		// CTRL-Zeichen ('\n', '\t' etc.) verwerfen

				// zum debuggen:
				//while(!(USR & (1<<UDRE)));		// Warten solange bis Zeichen gesendet wurde
				//UDR = eth_buffer[a];				// Ausgabe des Zeichens

				if (tcpsrv_status.read_argv) {
					tcpsrv_status.argvbuffer[tcpsrv_status.argvindex++] = eth_buffer[a];
					tcpsrv_status.argvbuffer[tcpsrv_status.argvindex] = 0;	// und gleich wieder terminieren
					if (tcpsrv_status.argvindex > MAXARGVBUFFER) {
						tcpsrv_status.argvindex= 0;			// Pufferberlauf
						tcpsrv_status.argvbuffer[0] = 0;
						tcpsrv_status.read_argv = 0;
						break;
					}
				}

				else {
					tcpsrv_status.cmdbuffer[tcpsrv_status.cmdindex++] = eth_buffer[a];
					tcpsrv_status.cmdbuffer[tcpsrv_status.cmdindex] = 0;	// und gleich wieder terminieren
					if (tcpsrv_status.cmdindex > MAXCMDBUFFER) {
						// TODO: Fehler melden

						tcpsrv_status.cmdindex= 0;			// cmd Puffer resetten
						tcpsrv_status.read_argv = 0;
						tcpsrv_status.cmdbuffer[0] = 0;
					}
				}
			}
		}

		TCPSRV_DEBUG("\r\nTCP Server-CTRL sende ACK");
		tcp_entry[index].status =  ACK_FLAG;	// empfangenes Paket besttigen
		create_new_tcp_packet(0,index);
		tcpsrv_status.ctl_waitack = 0;
		return;
	}

	// ist dies ein ACK von einem gesendeten Paket
	if ((tcp_entry[index].status&ACK_FLAG)) {	//&& (tcpsrv_status.ctl_waitack)
		tcp_entry[index].time = TCP_TIME_OFF;
		tcpsrv_status.ctl_waitack = 0;
		TCPSRV_DEBUG("\r\nTCP Server-CTRL ACK empfangen. wait -> %i", tcpsrv_status.ctl_waitack);

		//rx_buffer_pointer_out = retransmission_pointer;
		if (tcpsrv_status.send_226) {		// muss noch "226 Transfer Ende" gesendet werden?
			TCPSRV_DEBUG("\r\nTCP Server-CTRL sende 226");
			tcpsrv_status.send_226 = 0;
			respond_P(CTRL_CHANNEL, PSTR("226 Transfer complete. Closing data connection\r\n"),index);
		}
		return;
	}
	
	// Timeout kein ack angekommen
	if (tcp_entry[index].status == 0) {
		TCPSRV_DEBUG("\r\nTCP Server-CTRL TIMEOUT");
		//Daten nochmal senden
		tcpsrv_status.ctl_waitack = 0;
		//tcpsrv_send_data ();
		return;
	}
	
	return;
}

/*
*	Antwortstring(s) senden
*/
void respond_P(uint8_t channel, const char *pstr, unsigned char index)
{
	uint16_t len = strlen_P(pstr);
	memcpy_P(&eth_buffer[TCP_DATA_START],pstr,len);

	_respond(channel, len,index);
}


// AR das const for char *pstr rausgenommen
void respond(uint8_t channel, char *pstr, unsigned char index)
{
	uint16_t len = strlen(pstr);
	memcpy(&eth_buffer[TCP_DATA_START],pstr,len);
	_respond(channel, len,index);
}

void _respond(uint8_t channel, uint16_t len, unsigned char index)
{
	tcp_entry[index].status =  ACK_FLAG | PSH_FLAG;
	tcp_entry[index].time = TCP_TIME_OFF;
	create_new_tcp_packet(len,index);

	eth_buffer[TCP_DATA_START+len] = 0;	// nur fr debug
	TCPSRV_DEBUG("\r\nTCPSRV_DEBUG respond %i (%i): %s",index, len, &eth_buffer[TCP_DATA_START]);

	if (channel == CTRL_CHANNEL) {
		tcpsrv_status.ctl_waitack = 1;		// dieses Paket sollte besttigt werden
		tcpsrv_status.cmdindex= 0;			// auf erstes Zeichen setzen
		tcpsrv_status.cmdbuffer[0] = 0;		// und cmd-string terminieren
		tcpsrv_status.argvindex= 0;			// cmd Puffer resetten
		tcpsrv_status.argvbuffer[0] = 0;
		tcpsrv_status.read_argv = 0;
	}
	else
		tcpsrv_status.data_waitack = 1;		// dieses Paket sollte besttigt werden
}



void respond_226(unsigned char index)
{
	TCPSRV_DEBUG("\r\nTCP Server %i respond 226 wait: %i",index, tcpsrv_status.ctl_waitack);
	if (index < MAX_TCP_ENTRY) {
		if (tcpsrv_status.ctl_waitack)
			tcpsrv_status.send_226 = 1;			// nach nchstem ACK wird "226 Transfer Ende" gesendet
		else
			respond_P(CTRL_CHANNEL, PSTR("226 Transfer complete. Closing data connection\r\n"),index);
	}
}


